#ifndef _MINI3D_H_
#define _MINI3D_H_

#include "types.h"

enum {
	M3D_SMOOTH,
	M3D_LIGHTING,
	M3D_LIGHT0,
	M3D_LIGHT1,
	M3D_LIGHT2,
	M3D_LIGHT3,
	M3D_TEXTURE_2D,
	M3D_TEXTURE_CUBE,
	M3D_DEPTH_TEST,
	M3D_DEPTH_WRITE, /* set through m3d_depth_mask() rather than m3d_enable/disable */
	M3D_BLEND,
	M3D_LIGHT_MODEL_LOCAL_VIEWER,	/* set through m3d_light_model() */
	M3D_LIGHT_MODEL_AMBIENT,
	M3D_NORMALIZE,
	M3D_PHONG,	/* oh yeah :) */

	M3D_FLAT,	/* not a bit field */

	M3D_MODELVIEW = 100,
	M3D_PROJECTION,

	M3D_LINES = 200,
	M3D_TRIANGLES,
	M3D_QUADS,

	M3D_COLOR_BUFFER_BIT = 300,
	M3D_DEPTH_BUFFER_BIT,

	M3D_POSITION = 400,
	M3D_AMBIENT,
	M3D_DIFFUSE,
	M3D_AMBIENT_AND_DIFFUSE,
	M3D_SPECULAR,
	M3D_EMISSION,
	M3D_SHININESS,

	M3D_ZERO = 500,
	M3D_ONE,
	M3D_SRC_COLOR,
	M3D_ONE_MINUS_SRC_COLOR,
	M3D_DST_COLOR,
	M3D_ONE_MINUS_DST_COLOR,
	M3D_SRC_ALPHA,
	M3D_ONE_MINUS_SRC_ALPHA,
	M3D_DST_ALPHA,
	M3D_ONE_MINUS_DST_ALPHA,

	M3D_RGB = 700,
	M3D_RGBA,
	M3D_BGR,
	M3D_BGRA,
	
	M3D_UNSIGNED_BYTE = 800,
	M3D_UNSIGNED_SHORT
};

#ifdef __cplusplus
extern "C" {
#endif	/* __cplusplus */

void m3d_init(void);
void m3d_destroy(void);
void m3d_viewport(int x, int y, int w, int h);
uint32_t *m3d_get_pixel_data(void);

/* clear */
void m3d_clear_color(float r, float g, float b, float a);
void m3d_clear_depth(float d);
void m3d_clear(unsigned int what);

/* general state */
void m3d_enable(unsigned int what);
void m3d_disable(unsigned int what);
void m3d_light_model(unsigned int pname, int val);
void m3d_light_modelv(unsigned int pname, float *param);
void m3d_blend_func(unsigned int src, unsigned int dst);
void m3d_shade_model(unsigned int mode);

/* zbuffer state */
void m3d_depth_mask(int boolval);

/* lights and materials */
void m3d_light(unsigned int light, unsigned int pname, float *params);
void m3d_material(unsigned int pname, float param);
void m3d_materialv(unsigned int pname, float *params);

/* texture state */
void m3d_gen_textures(int n, unsigned int *tex);
void m3d_delete_textures(int n, unsigned int *tex);
int m3d_is_texture(unsigned int tex);
void m3d_bind_texture(unsigned int targ, unsigned int tex);
void m3d_tex_image(unsigned int targ, int lvl, int ifmt, int w, int h, int border, unsigned int fmt, unsigned int type, const void *pixels);

/* matrix manipulation */
void m3d_matrix_mode(unsigned int mode);
void m3d_load_identity(void);
void m3d_load_matrix(float *mat);
void m3d_mult_matrix(float *mat);

void m3d_push_matrix(void);
void m3d_pop_matrix(void);

void m3d_translate(float x, float y, float z);
void m3d_rotate(float angle, float x, float y, float z);
void m3d_rotate_euler(float x, float y, float z);
void m3d_scale(float x, float y, float z);

void m3d_perspective(float fovy, float aspect, float znear, float zfar);

/* rendering */
void m3d_begin(unsigned int primitive);
void m3d_end(void);

void m3d_vertex(float x, float y, float z);
void m3d_color3(float r, float g, float b);
void m3d_color4(float r, float g, float b, float a);
void m3d_normal(float x, float y, float z);
void m3d_tex_coord(float u, float v);

#ifdef __cplusplus
}
#endif	/* __cplusplus */

#endif	/* _MINI3D_H_ */
